# Copyright (C) 2016- Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import pprint


def application(environ, start_response):
    status = '404 Not Found'
    start_response(status, [('content-type', 'text/plain')])
    s = environ['statsd']
    statsd = 'statsd: {} {}'.format(s._addr, s._prefix)
    output = (status + '\n\n' +
              statsd + '\n\n' +
              pprint.pformat(environ))

    return output.encode('utf8')
